#!/usr/bin/python2.7
# -*- coding: utf-8 -*-

#  Copyright (C) 2014-2015  Rafael Senties Martinelli <rsm@imap.cc>
#
#  This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License 3 as published by
#   the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.

"""
	To Fix:

	- Resize the waiting_window after the expander is collapsed ( this is kinda like a bug )
	- The progress bar seems to have some little bugs..


	To Implement:

	- Auto install .deb packages, or add custom command to install packages

	- Hide the folders from the folder chooser when they've alredy been selected.
		(This feature must be requested to the GTK developers)

"""

import os, csv, time, getpass, re, psutil, numpy, threading, subprocess, sys
from multiprocessing import Process, Queue, current_process, freeze_support
from gi.repository import Gtk, GObject, GLib, Gdk
from uuid import getnode as get_mac
from os import stat
from inspect import currentframe, getframeinfo

## Imports from other files of Fresh Install
sys.path.insert(0, 'DataFunctions/')
sys.path.insert(0, 'GeneralFunctions/')
sys.path.insert(0, 'GTK/')

	# From DataFunctions /
from FilesToRename import *
from FilesToReplace import *
from FoldersToReplace import *
from ProgramsToInstall import *
from Messages import *
from Paths import *

	# From GeneralFunctions/
from CustomOs import *; CustomOs=CustomOs()
from CustomGtk import *; CustomGtk=CustomGtk()
from CustomCsv import *; CustomCsv=CustomCsv()
from CustomParser import *; CustomParser=CustomParser()
from CustomTextFile import *; CustomTextFile=CustomTextFile()

	# From GTK/
from Glade_Builder_Import import *
from VTE_Terminal import *
from TreeViews import *
from ComboBox import *
from Labels import *


def DELETE_program_to_install(widget, self):
	
	(model, rownumbers) = self.treeview_ProgramsToInstall_selection.get_selected_rows()	
	
	## Delete the items from the liststore
	iters = []
	for rownumber in rownumbers:
		iters.append(self.ListStore_ProgramsToInstall.get_iter(rownumber))
		
		
	## Delete the programs from the ram data
	programs_to_remove=[]
	for i in iters:
		if i is not None:
			apt_name=str(self.ListStore_ProgramsToInstall[i][1])

			for program in self.ProgramsToInstall_List:
				if program.get_apt_name() == apt_name:				
					programs_to_remove.append(program)
					break
				
				
		self.ListStore_ProgramsToInstall.remove(i)
				
	for program in programs_to_remove:	  
		self.ProgramsToInstall_List.remove(program)
	
	## Overwrite the new list to the csv file
	with open(PROGRAMS_TO_INSTALL_CSV.format(self.Data_Location), 'w') as f:
		for program in self.ProgramsToInstall_List:
			csv_list=csv.writer(f,delimiter='|')
			csv_list.writerow( [program.get_apt_name(), program.get_category() , program.get_description(), program.get_comment()] )
			
			
	## Actualize the categories of the comboboxes
	POPULATE_Combobox_get_category(self)
	POPULATE_Combobox_Category2(self)


def DELETE_files_to_rename(widget, self):
	(model, rownumbers) = self.treeview_FilesToRename_selection.get_selected_rows()	
	
	## Delete the items from the liststore
	iters = []
	for row in rownumbers:
		iters.append(self.ListStore_FilesToRename.get_iter(row))
	
	## Delete the programs from the ram
	files_to_remove=[]
	for i in iters:
		if i is not None:
			path=str(self.ListStore_FilesToRename[i][0])

			for file in self.classlist_files_to_rename:
				if file.get_computer_path() == path:
					files_to_remove.append(file)
					break
					
			self.ListStore_FilesToRename.remove(i)
	
	for file in files_to_remove:
		self.classlist_files_to_rename.remove(file)
	
	
	## Overwrite the new list to the csv file
	with open(FILES_TO_RENAME_CSV.format(self.Data_Location), 'w') as f:
		for file in self.classlist_files_to_rename:
			csv_list=csv.writer(f,delimiter='|')
			csv_list.writerow([ file.get_computer_path(), file.get_new_name(), file.get_computer_date(), 
								file.get_computer_path(), file.get_computer_size() ])

def DELETE_files_to_replace(widget, self):
	(model, rownumbers) = self.treeview_FilesToReplace_selection.get_selected_rows()	
	
	
	## Delete the items from the liststore
	iters = []
	for row in rownumbers:
		iters.append(self.ListStore_FilesToReplace.get_iter(row))
	
	## Delete the programs from the ram
	files_to_remove=[]
	for i in iters:
		if i is not None:
			path=str(self.ListStore_FilesToReplace[i][0])

			for file in self.classlist_files_to_replace:
				if file.get_computer_path() == path:
					files_to_remove.append(file)
					break
					
			self.ListStore_FilesToReplace.remove(i)
	
	for file in files_to_remove:
		self.classlist_files_to_replace.remove(file)
	
	## Overwrite the new list to the csv file
	with open(FILES_TO_REPLACE_CSV.format(self.Data_Location), 'w') as f:
		for file in self.classlist_files_to_replace:
			csv_list=csv.writer(f,delimiter='|')
			csv_list.writerow( [file.get_computer_path(), file.get_computer_path(), file.get_id()] )
			
		
	## Delete the stored folders
	for file in files_to_remove:
		CustomOs.DeleteFromDisk(file.get_stored_path())
	
def DELETE_folders_to_replace(widget, self):
	(model, rownumbers) = self.treeview_FoldersToReplace_selection.get_selected_rows()	
	
	## Delete the items from the liststore
	iters = []
	for row in rownumbers:
		iters.append(self.ListStore_FoldersToReplace.get_iter(row))
		
		
	folders_to_remove=[]
	## Delete the programs from the ram
	for i in iters:
		if i is not None:
			path=str(self.ListStore_FoldersToReplace[i][0])

			for folder in self.classlist_folders_to_replace:
				if folder.get_computer_path() == path:
					folders_to_remove.append(folder)
					break
					
		self.ListStore_FoldersToReplace.remove(i)

	for folder in folders_to_remove:	  
		self.classlist_folders_to_replace.remove(folder)
	
	## Overwrite the new list to the csv folder
	with open(FOLDERS_TO_REPLACE_CSV.format(self.Data_Location), 'w') as f:
		for folder in self.classlist_folders_to_replace:
			csv_list=csv.writer(f,delimiter='|')
			csv_list.writerow( [folder.get_computer_path(), folder.get_computer_path(), folder.get_id()] )
			
	
	## Delete the stored folders
	for folder in folders_to_remove:
		CustomOs.DeleteFromDisk(folder.get_stored_path())
	



def RSYNC(self,source,destination,GUI):
	
	if not source or source==None:
		print "fresh-install.py Error: RSYNC No Source"
		
	elif not destination or destination==None:
		print "fresh-install.py Error: RSYNC No Destination"
	
	elif GUI==False:
		os.system('''rsync -av --delete --progress "{0}" "{1}"'''.format(source,destination))

	elif GUI==True:
		
		local_progress_file=TMP_SYNC_PROGRESS.format(self.Current_User)
		
		if os.path.exists(TMP_INSTALLATION_PROGRESS.format(0)):
			os.remove(TMP_INSTALLATION_PROGRESS.format(0))
		
		
		if os.path.exists(local_progress_file):
			os.remove(local_progress_file)
		
		CustomOs.MakeTreeOfDirs(os.path.dirname(destination))


		command='''rsync -av --delete --progress "{src}" "{dst}" | tee {prog}; echo " "; echo " " > {ended}'''.format(	src=source, 
																														dst=destination, 
																														prog=local_progress_file,
																														ended=TMP_INSTALLATION_PROGRESS.format(0))
		

		command_on_VTE(self,'''echo " "''') 
		command_on_VTE(self,'''echo "----- New Transfer -----"''') 
		command_on_VTE(self,'''echo "source: {src}"'''.format(src=source))
		command_on_VTE(self,'''echo "destination: {dst} "'''.format(dst=destination))

		Process(target=command_on_VTE, args=[self, command]).start()

		progress=self.Synchronized_Size/self.Size_To_Actualize
		if self.Size_To_Actualize > 0:
			""" 
			 	This updates the progress bar when files are not created,
				another way of doing this it would be to get the node id and read the node size. Would it be better?
			"""
			while True:
				
				if os.path.exists(destination) and CustomOs.GetPathSize_KB(destination) > 90:
					progress=(self.Synchronized_Size+CustomOs.GetPathSize_KB(destination))/self.Size_To_Actualize
				else:
					""" 
						When a file does not exists, read the 
						rsync log in order to know the progress of the current file.
					"""
					if os.path.exists(local_progress_file):
						value=False
						with open(local_progress_file,'rU') as f:
							lines = f.readlines()
							max=0
							for line in lines:
								if "%" in line:
									try:
										value=line.split()[0]
										value=int(value)
										if value and value > max:
											max=value
									
									except Exception,e:
										if not 'invalid literal for int' in str(e):
											print frameinfo.filename, frameinfo.lineno
											print str(e)
										
									
						if value:	
							self.Synchronized_Size=((max/(2**30.0000))+self.Synchronized_Size)
							progress=self.Synchronized_Size/self.Size_To_Actualize
				
				"""
					This prevents that the progress bar displays 100% before all the thread be finishd.
				"""
				if progress > 0 and progress <= 0.995:
					GLib.idle_add(self.progressbar.set_fraction, progress)
				
				
				if os.path.exists(TMP_INSTALLATION_PROGRESS.format(0)):
					break
				
				time.sleep(0.5)
					
			#####################
			try:
				self.Synchronized_Size=self.Synchronized_Size+CustomOs.GetPathSize_KB(destination)	
				progress=self.Synchronized_Size/self.Size_To_Actualize
			
			except Exception,e: 
				frameinfo = getframeinfo(currentframe())
				print frameinfo.filename, frameinfo.lineno
				print str(e)
				print "Error on CustomOs.GetPathSize_MB, if a transfer was stopped ignore this warning."
			
			if progress > 0 and progress <= 0.995:
				GLib.idle_add(self.progressbar.set_fraction,progress)
		
		
	Actualize_FoldersToReplace(self)
	Actualize_FilesToReplace(self)
	


def LOAD_external_config(self,path):
	
		if CustomParser.check_value(path,'fresh-install_configuration','program_Data_Location'):
			self.Data_Location=CustomParser.get_value(path,'fresh-install_configuration','program_Data_Location')
			if not os.path.exists(PROGRAM_DATA.format(self.Data_Location)):
				parent_path=os.path.dirname(path)
				if os.path.exists(PROGRAM_DATA.format(parent_path)):
					self.Data_Location=parent_path
				elif os.path.exists(parent_path+PROGRAM_DATA.format(main_folder)):
					self.Data_Location=parent_path
		else:
			self.Data_Location=os.path.dirname(os.path.dirname(path))

		if CustomParser.get_value(path,'fresh-install_configuration','Merged_configurations')=="true":
			self.Parser_Section='fresh-install_configuration'
		else:
			self.Parser_Section='fresh-install_'+self.Current_User+'_Configuration'	
		
		self.Path_File_Config=path
		CustomParser.write_value(self.Path_File_Config,'fresh-install_configuration','program_Data_Location',self.Data_Location)

		Configuration.init_gui(self)

	
def REFRESH_installed_programs(self, skip_verifycation):
	

	if CustomGtk.ListStoreNbRows(self.ListStore_ProgramsToInstall) <= 0 and not skip_verifycation:
		Gdk.threads_enter()
		CustomGtk.info(self.window_install_programs,TEXT_NO_PROGRAM_TO_INSTALL,TEXT_YOU_NEED_TO_ADD_PROGRAMS)
		Gdk.threads_leave()
	else:
		## Get the list of the installed programs
		installed_programs=os.popen("aptitude search '~i!~M' -F '%p'")
		installed_programs=str(installed_programs.read()).split("\n")
		
		## Update installed instance of the programs list
		for stored_program in self.ProgramsToInstall_List:
			for program in installed_programs:
				stored_program.set_installed("No")
				if program.strip() == stored_program.get_apt_name():
					stored_program.set_installed("Yes")
					break

		## Update the window of the user
		i=0
		for row in self.ListStore_ProgramsToInstall:
			for program in self.ProgramsToInstall_List:
				if self.ListStore_ProgramsToInstall[i][1] == program.get_apt_name():
					self.ListStore_ProgramsToInstall[i][0]=program.get_installed()
			i+=1


def INSTALL_programs(self):

	text='''The program uses multiple lines of 'apt-get install' because the option '--ignore-missing' has a bug, so if a package fails all the installation would fail.'''
	
	command_on_VTE(self,'''echo " " '''.format(text))
	command_on_VTE(self,'''echo "{0}" '''.format(text))

	"""
		Create the apt-get command and send it.
	"""
	box_category=CustomGtk.retrive_combobox_value(self.combobox_category2,False)
	
	# get the number of programs to install
	programs_to_install=0
	if box_category==TEXT_SELECT_ALL_THE_CATEGORIES:
		for program in self.ProgramsToInstall_List:
			programs_to_install+=1

	else:
		for program in self.ProgramsToInstall_List:
			if program.get_category() ==  box_category:
				programs_to_install+=1
	
	
	# remove the possible tmp files
	for i in range(1,programs_to_install+1):
		path=TMP_INSTALLATION_PROGRESS.format(i)
		if os.path.exists(path):
			os.remove(path)
		i+=1
	
	
	# install the programs
	i=0
	if box_category==TEXT_SELECT_ALL_THE_CATEGORIES:
		for program in self.ProgramsToInstall_List:
			i+=1
			command_on_VTE(self,'''echo " "''')
			command_on_VTE(self,'''apt-get --yes install {0} ; echo  " " > "{1}" '''.format(program.get_apt_name(),TMP_INSTALLATION_PROGRESS.format(i)))
			update_the_installation_progresss(self,i,programs_to_install)
	else:
		for program in self.ProgramsToInstall_List:
			if program.get_category() ==  box_category:
				i+=1
				command_on_VTE(self,'''echo " "''')
				command_on_VTE(self,'''apt-get --yes install {0} ; echo  " " > "{1}" '''.format(program.get_apt_name(),TMP_INSTALLATION_PROGRESS.format(i)))
				update_the_installation_progresss(self,i, programs_to_install)
	
	REFRESH_installed_programs(self, True)


def update_the_installation_progresss(self, i, programs_to_install):
	"""
		update the installation progress and prevent that it tries to install all the programs and the same time
	"""	
	while True:
		if os.path.exists(TMP_INSTALLATION_PROGRESS.format(i)):
			
			installation_progress=(i+0.00)/programs_to_install
			
			if installation_progress <= 0.995:
				GLib.idle_add(self.progressbar.set_fraction, installation_progress)
		
			break

		time.sleep(0.5)
	
	
	

def GENERATE_installation_bash_script(self,path):
	"""
		This method is the fix of GENERATE_installation_bash_script_FROM_THE_PREVIOUS_VERSIONS
	"""

 
 	header='''#!/bin/bash

 # Generated with Fresh Install: rsm.imap.cc/software/fresh-install.html
	
		#
		# The script is using multiple lines of apt-get install because
		# the option '--ignore-missing' of 'apt-get' has a bug, so if a package fails
		# all the installation would fail..
		#
	
 if [ "$(id -u)" != "0" ]; then
	echo "You need root permision for installing packages." 1>&2
	exit 1
 fi	
 
\n'''

	command='''	apt-get --yes install {}\n'''
 

	box_category=CustomGtk.retrive_combobox_value(self.combobox_category2,False)
	
	# Set the name of the file
	if box_category==TEXT_SELECT_ALL_THE_CATEGORIES:
		bash_file=GENERATED_BASH.format(path)
	else:
		bash_file=GENERATED_BASH_filtred.format(path,box_category)

	# Remove a previous file
	if os.path.exists(bash_file):
		os.remove(bash_file)

	
	# Write the new file
	with open(bash_file, 'w') as f:
		f.write(header)
		
		if box_category==TEXT_SELECT_ALL_THE_CATEGORIES:
			for program in self.ProgramsToInstall_List:
				f.write(command.format(program.get_apt_name()))
		else:
			for program in self.ProgramsToInstall_List:
				if program.get_category() ==  box_category:
					f.write(command.format(program.get_apt_name()))
	
		
	# Allow the file to be excecuted
	os.system("chmod a+x '"+bash_file+"'")
	

def REPLACE_folders(self,skip_dialog):

	error_list=[]														# TO FINISH. Add an error category?
	entry_replace_filter=self.entry_replace_filter.get_text()
	
	
	if self.warning_replacing_folders and not skip_dialog:
		Gdk.threads_enter()
		if not CustomGtk.question(self.window_replace,TEXT_WARNING,TEXT_WILL_REPLACE_FOLDERS_WANT_TO_PROCEED):
			Gdk.threads_leave()
			return
		Gdk.threads_leave()
	
	
	for folder in self.classlist_folders_to_replace:
		
		folder_computer_path=folder.get_computer_path()
		folder_stored_path=folder.get_stored_path()+"/"
		

		if os.path.exists(folder_computer_path):
			check_path=folder_computer_path
		else:
			check_path=folder_stored_path
			
			
		if not os.path.exists(folder_stored_path):
			error_list.append(folder)
			
		elif self.checkbutton_replace_if.get_active():
				if entry_replace_filter in folder_computer_path:
					if CustomTextFile.CurrentUserCanWrite(folder_computer_path):
						folder_computer_path=replace_strings_in_path(self,folder_computer_path)
						RSYNC(self,folder.get_stored_path()+"/",folder_computer_path,True)
					else:
						error_list.append(folder)
		
		elif self.checkbutton_replace_if_not.get_active():
			if not entry_replace_filter in folder_computer_path:
				if CustomTextFile.CurrentUserCanWrite(folder_computer_path):
					folder_computer_path=replace_strings_in_path(self,folder_computer_path)
					RSYNC(self,folder.get_stored_path()+"/",folder_computer_path,True)
				else:
					error_list.append(folder)

		else:
			if CustomTextFile.CurrentUserCanWrite(check_path):
				folder_computer_path=replace_strings_in_path(self,folder_computer_path)
				RSYNC(self,folder.get_stored_path()+"/",folder_computer_path,True)
			else:
				error_list.append(folder)


def RENAME_files(self):
	
	text_filter=self.entry_rename_filter.get_text()
	
	if self.checkbutton_rename_if.get_active() and text_filter.strip():
		for file in self.classlist_files_to_rename:
			if os.path.exists(file.get_computer_path()) and text_filter in file.get_computer_path():
				os.rename(file.get_computer_path(), file.get_new_path())
				
	elif self.checkbutton_rename_if_not.get_active() and text_filter.strip():
		for file in self.classlist_files_to_rename:
			if os.path.exists(file.get_computer_path()) and not text_filter in file.get_computer_path():
				os.rename(file.get_computer_path(), file.get_new_path())
	else:
		for file in self.classlist_files_to_rename:
			if os.path.exists(file.get_computer_path()):
				
				print file.get_computer_path()
				print file.get_new_path()
				
				os.rename(file.get_computer_path(), file.get_new_path())
	

def GET_SIZE_REPLACE_folders(self):

	entry_replace_filter=self.entry_replace_filter.get_text()
	SIZE=0.0
	
	for folder in self.classlist_folders_to_replace:
		
		if os.path.exists(folder.get_computer_path()):
			check_path=folder.get_computer_path()
		else:
			if os.path.exists(folder.get_stored_path()):
				check_path=folder.get_stored_path()
			else:
				check_path=False
				
		if check_path:
			if self.checkbutton_replace_if.get_active():
				for folder in self.classlist_folders_to_replace:
					if entry_replace_filter in folder.get_computer_path():
						if CustomTextFile.CurrentUserCanWrite(check_path):
							SIZE=SIZE+CustomOs.GetPathSize_KB(folder.get_stored_path())

			elif self.checkbutton_replace_if_not.get_active():
				if not entry_replace_filter in folder_path:
					if CustomTextFile.CurrentUserCanWrite(check_path):
						SIZE=SIZE+CustomOs.GetPathSize_KB(folder.get_stored_path())

			else:
				for folder in self.classlist_folders_to_replace:
					if CustomTextFile.CurrentUserCanWrite(check_path):
						SIZE=SIZE+CustomOs.GetPathSize_KB(folder.get_stored_path())

	return float(SIZE)
		
	
def REPLACE_files(self, skip_dialog):

	error_list=[]														# TO FINISH. Add an error category?
	entry_replace_filter=self.entry_replace_filter.get_text()
	
	if self.warning_replacing_files and not skip_dialog:

		Gdk.threads_enter()
		if not CustomGtk.question(self.window_replace,TEXT_WARNING,TEXT_WILL_REPLACE_FILES_WANT_TO_PROCEED):
			Gdk.threads_leave()
			return
		Gdk.threads_leave()
		

	for file in self.classlist_files_to_replace:
		
		file_computer_path=file.get_computer_path()
		file_stored_path=file.get_stored_path()
				
		if os.path.exists(file_computer_path):
			check_path=file_computer_path
		else:
			check_path=file_stored_path

		if not os.path.exists(file_stored_path):
			error_list.append(file)
			#TEXT_THE_FILE_IS_MISSING
		
		if self.checkbutton_replace_if.get_active():
			if entry_replace_filter in file_computer_path:
				if CustomTextFile.CurrentUserCanWrite(check_path):
					file_computer_path=replace_strings_in_path(self,file_computer_path)
					RSYNC(self,file_stored_path,file_computer_path,True)
				else:
					error_list.append(file)
					#TEXT_WRITE_PERMISSION_PROBLEM
				
		elif self.checkbutton_replace_if_not.get_active():
			if not entry_replace_filter in file_computer_path:
				if CustomTextFile.CurrentUserCanWrite(check_path):
					file_computer_path=replace_strings_in_path(self,file_computer_path)
					RSYNC(self,file_stored_path,file_computer_path,True)
				else:
					error_list.append(file)
					#TEXT_WRITE_PERMISSION_PROBLEM
					
		else:
			if CustomTextFile.CurrentUserCanWrite(check_path):
				file_computer_path=replace_strings_in_path(self,file_computer_path)
				RSYNC(self,file_stored_path,file_computer_path,True)
			else:
				error_list.append(file)
			#error=TEXT_WRITE_PERMISSION_PROBLEM


def GET_SIZE_REPLACE_files(self):

	entry_replace_filter=self.entry_replace_filter.get_text()
	SIZE=0.0
	
	for file in self.classlist_files_to_replace:
		
		if os.path.exists(file.get_computer_path()):
			check_path=file.get_computer_path()
		else:
			if os.path.exists(file.get_stored_path()):
				check_path=file.get_stored_path()
			else:
				check_path=False
			
		if check_path:
			if self.checkbutton_replace_if.get_active():
				for file in self.classlist_files_to_replace:
					if entry_replace_filter in file.get_computer_path():
						if CustomTextFile.CurrentUserCanWrite(check_path):
							SIZE=SIZE+CustomOs.GetPathSize_KB(file.get_stored_path())

			elif self.checkbutton_replace_if_not.get_active():
				if not entry_replace_filter in file_path:
					if CustomTextFile.CurrentUserCanWrite(check_path):
						SIZE=SIZE+CustomOs.GetPathSize_KB(file.get_stored_path())

			else:
				for file in self.classlist_files_to_replace:
					if CustomTextFile.CurrentUserCanWrite(check_path):
						SIZE=SIZE+CustomOs.GetPathSize_KB(file.get_stored_path())

	return float(SIZE)


def ACTUALIZE_files_and_folders(self):

	Gdk.threads_enter()
	self.window_root.get_window().set_cursor(self.watch_cursor)
	Gdk.threads_leave()
	Gdk.threads_enter()
	self.button_actualize.set_sensitive(False)
	Gdk.threads_leave()
	
	self.Size_To_Actualize=GET_SIZE_ACTUALIZE_files_and_folders(self)
	CustomOs.MakeTreeOfDirs(COMPUTER_DATA.format(self.Data_Location))


	for folder in self.classlist_folders_to_replace:
		if os.path.exists(folder.get_computer_path()):
			if CustomTextFile.CurrentUserCanWrite(folder.get_computer_path()):
				RSYNC(self, folder.get_computer_path(), folder.get_stored_path(), True)


	for file in self.classlist_files_to_replace:
		if os.path.exists(file.get_computer_path()):
			if CustomTextFile.CurrentUserCanWrite(file.get_computer_path()):
				RSYNC(self, file.get_computer_path(), file.get_stored_path(), True)
	
	CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'last_actualization',CustomOs.GetDateTime())
	GLib.idle_add(self.label_act.set_label,"")
	
	Gdk.threads_enter()
	self.window_root.get_window().set_cursor(None)
	Gdk.threads_leave()
	Gdk.threads_enter()
	self.button_actualize.set_sensitive(True)
	Gdk.threads_leave()


def GET_SIZE_ACTUALIZE_files_and_folders(self):
	
	size=0.0
	#
	# The size is read from the computer and not from the data of the Classes,
	# In case the files has been modified/updated.
	#
	for folder in self.classlist_folders_to_replace:
		if os.path.exists(folder.get_computer_path()):
			size=size+CustomOs.GetPathSize_KB(folder.get_computer_path())
					
	for file in self.classlist_files_to_replace:			
		if os.path.exists(file.get_computer_path()):
			size=size+CustomOs.GetPathSize_KB(file.get_computer_path())
	
	return float(size)

	
def DELETE_all_stored_data(self):
	
	continue_on_dialog=False
	program_data=PROGRAM_DATA.format(self.Data_Location)
	computer_data=COMPUTER_DATA.format(self.Data_Location)
	
	
	if os.path.exists(program_data) or os.path.exists(computer_data):
		"""
			Get the size of the stored data to prevent the user
		"""
		if os.path.exists(computer_data):
			computer_data_size=CustomOs.GetPathSize_GB(computer_data)
		else:
			computer_data_size=0
		if os.path.exists(program_data):	
			program_data_size=CustomOs.GetPathSize_GB(program_data)
		else:
			program_data_size=0
		total_data=computer_data_size+program_data_size
		
		"""
			prevent the user
		"""
		if total_data >= 0.1:
			continue_on_dialog=CustomGtk.question(self.window_root,TEXT_YOU_HAVE_STORED_X_GB_WANT_DELETE.format(round(total_data,2)),None)
		else:
			continue_on_dialog=CustomGtk.question(self.window_root,TEXT_SOME_DATA_STORED_WANT_TO_DELETE,None)
	
	elif not os.path.exists(program_data) and not os.path.exists(computer_data) and os.path.exists(self.Path_File_Config):
		"""
			if the only information is the configuration file..
		"""
		continue_on_dialog=CustomGtk.question(self.window_root,TEXT_THE_ONLY_FILE_IS_THE_CONFIGURATION,None)
			
			
	if continue_on_dialog:
		"""
			Delete the data and restart the GUI
		"""
		
		if not CustomTextFile.CurrentUserCanWrite(self.Data_Location):
			CustomGtk.info(self.window_root,TEXT_YOU_DONT_HAVE_PERMISSION_TO_DELETE_ALL_THE_DATA,None)
			return

		Thread.set_watch_cursor(self,self.window_root,self.button_reset,delete_data_on_watch_cursor,[self])

		LABEL_last_actualization(self)
	
		self.window_setup.set_transient_for(self.window_root)
		self.window_root.hide()
		self.window_setup.show()
		#Configuration.init_gui(self)
		#Configuration.init_session(self)
		
		
		

def check_if_apt_is_locked(self,parent,message):
	"""Check if apt is locked"""
	
	locked1,locked2,choice=False,True,False
	while (locked1 or locked2):
		
		if os.path.exists(TEMP_APT_LOCKED1):
			os.remove(TEMP_APT_LOCKED1)
		if os.path.exists(TEMP_APT_LOCKED2):
			os.remove(TEMP_APT_LOCKED2)
		
		command_on_VTE(self,'''apt-get install > {0} \n'''.format(TEMP_APT_LOCKED2))				
		
		while not os.path.exists(TEMP_APT_LOCKED2):
			time.sleep(0.1)
		#os.system("pkill apt-locked")
		with open('/tmp/fresh-install_apt-locked2', 'r') as f:
			locked2=f.read()
			if "Unable to lock the administration directory" in locked2:
				choice=DIALOG_question(parent,message,None)
				if choice:
					if os.path.exists("/var/lib/apt/lists/lock"):
						os.remove("/var/lib/apt/lists/lock")
					if os.path.exists("/var/cache/apt/archives/lock"):
						os.remove("/var/cache/apt/archives/lock")
				else:
					return True,True,choice
			else:
				return False,False,choice

		#locked1=os.popen("ps -e | grep -e apt -e adept | grep -v grep")
		#locked1=str(locked1.read())
		#locked_output=locked1
		#print locked1
		#if "apt" in locked1:
		#	locked1=False
		#else:
		#	locked1=False
	
		#if locked1:
		#	choice=DIALOG_question(self.window_install_programs,TEXT_SEEMS_LIKE_APT_IS_LOCKED.format(locked_output),None)
		#	if choice:
		#		None
		#	else:
		#		return


def delete_data_on_watch_cursor(self):

	if os.path.exists(self.Data_Location):
		CustomOs.DeleteFromDisk(self.Data_Location)

	if os.path.exists(self.Path_File_Config):
		os.remove(self.Path_File_Config)
		
	while os.path.exists(self.Data_Location):
		time.sleep(0.1)


def CLEAR_gui_add_programs_to_install(self):
	self.entry_program_name.set_text("")
	self.entry_program_description.set_text("")
	self.entry_program_comment.set_text("")

def CREATE_FOLDER_tmp(self):
	if not os.path.exists(TEMP_DATA.format(self.Data_Location)):
		os.makedirs(TEMP_DATA.format(self.Data_Location))
		CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'mac_adress',get_mac())

def ADD_files_to_rename(self, file_path, new_name):
	
	CustomOs.MakeTreeOfDirs(PROGRAM_DATA.format(self.Data_Location))

	new_file=FilesToRename(file_path, new_name, self.Data_Location)
	
	self.classlist_files_to_rename.append(new_file)
	self.ListStore_FilesToRename.append([ new_file.get_computer_path(), new_file.get_new_name(), 
											new_file.get_computer_date(), new_file.get_computer_path(),
											new_file.get_computer_size() ])
	
	os.rename(file_path, new_file.get_new_path())

	CustomCsv.add_row(FILES_TO_RENAME_CSV.format(self.Data_Location), file_path, new_file.get_new_path())

def ADD_files_to_replace(self, file_path):

	list_path=FILES_TO_REPLACE_CSV.format(self.Data_Location)
	file_size=CustomOs.GetPathSize_GB(file_path)

	if not filter_files(file_path, self):
		CustomGtk.info(self.window_root,TEXT_FILE_ALREADY_SELECTED,None)
	else:
		if not filter_files_including_folders(file_path, self):
			dialog_value=CustomGtk.question(self.window_root,TEXT_FILE_STORED_IN_FOLDER_TO_REPLACE,TEXT_DO_YOU_STILL_WANT_TO_ADD_THE_FILE)
		else:
			if file_size >= (self.warning_file_size/1024.0):
				if not CustomGtk.question(self.window_root,TEXT_THE_FILE_SIZE_IS.format(round(file_size,2)),TEXT_DO_YOU_WANT_TO_ADD_IT):
					return
			

			if not os.path.exists(list_path):
				CustomOs.MakeTreeOfDirs(PROGRAM_DATA.format(self.Data_Location))
				CustomOs.MakeTreeOfDirs(COMPUTER_DATA.format(self.Data_Location))

			new_file=FilesToReplace(file_path, CustomCsv.increment_number_of_column(list_path,2), self.Data_Location)
			self.classlist_files_to_replace.append(new_file)
			self.ListStore_FilesToReplace.append([new_file.get_computer_path(), new_file.get_computer_date(), new_file.get_stored_date(),
			                                      new_file.get_computer_path(), new_file.get_stored_owner(),
												  str(new_file.get_computer_size()), str(new_file.get_stored_size())])


			CustomCsv.add_row(list_path,file_path, new_file.get_computer_path(), new_file.get_id())
			CustomCsv.alph_organise_file(list_path)
			
			if file_size >= self.warning_waiting_dialog:
				self.Size_To_Actualize=file_size
				Thread.start_waiting(self,self.window_root,RSYNC,(self,file_path, new_file.get_stored_path(),True))
			else:
				Thread.set_watch_cursor(self,self.window_root,self.button_files_to_replace,RSYNC,(self,file_path, new_file.get_stored_path(), False))

def ADD_folders_to_replace(self,folder_path):

	list_path=FOLDERS_TO_REPLACE_CSV.format(self.Data_Location)
	folder_size=CustomOs.GetPathSize_GB(folder_path)
	
	if not filter_folders(self, folder_path, None):
		CustomGtk.info(self.window_root,TEXT_FOLDER_ALREADY_SELECTED,None)
		
	else:
		if not filter_folders_included_in_folders(self, folder_path, None):
			if not CustomGtk.question(self.window_root, TEXT_FOLDER_ALREADY_IN_FOLDER, None):
				return
			
		
		if CustomOs.GetPathSize_GB(folder_path) >= self.warning_folder_size:
			if not CustomGtk.question(self.window_root, TEXT_THE_FOLDER_SIZE_IS.format(round(folder_size,2)),TEXT_DO_YOU_WANT_TO_ADD_IT):
				return
			
			
		if not os.path.exists(list_path):
			CustomOs.MakeTreeOfDirs(PROGRAM_DATA.format(self.Data_Location))
			CustomOs.MakeTreeOfDirs(COMPUTER_DATA.format(self.Data_Location))
			
	
		new_folder=FoldersToReplace(folder_path, CustomCsv.increment_number_of_column(list_path,2), self.Data_Location)
		self.classlist_folders_to_replace.append(new_folder)
		self.ListStore_FoldersToReplace.append([	new_folder.get_computer_path(), 
													new_folder.get_computer_date(), 
													new_folder.get_stored_date(),
													new_folder.get_computer_path(), 
													new_folder.get_stored_owner(),
													str(new_folder.get_computer_size()), 
													str(new_folder.get_stored_size())])
		
		
		CustomCsv.add_row(	list_path,
							folder_path, 
							new_folder.get_computer_path(), 
							new_folder.get_id())
		
		CustomCsv.alph_organise_file(list_path)
			
		if folder_size >= self.warning_waiting_dialog:
			self.Size_To_Actualize=folder_size
			Thread.start_waiting(self,self.window_root,RSYNC,(self, new_folder.get_computer_path(), new_folder.get_stored_path(),True))
		else:
			Thread.set_watch_cursor(self,self.window_root,self.button_folders_to_replace,RSYNC,(self, new_folder.get_computer_path(), new_folder.get_stored_path(),False))


def ADD_programs_to_install(self):
	
	category=CustomGtk.retrive_combobox_value(self.combobox_category,True)
	programs_to_install_path=PROGRAMS_TO_INSTALL_CSV.format(self.Data_Location)	
	
	if category==TEXT_SELECT_LIST_ALL_THE_CATEGORIES:
		Gdk.threads_enter()
		CustomGtk.info(self.window_add_programs,TEXT_MUST_SELECT_A_CATEGORY,TEXT_X_IS_NOT_A_CATEGORY.format(TEXT_SELECT_LIST_ALL_THE_CATEGORIES))
		Gdk.threads_leave()
	else:
		if not self.entry_program_name.get_text():
			Gdk.threads_enter()
			CustomGtk.info(self.window_add_programs,TEXT_PROGRAM_NAME_CANT_BE_EMPTY,None)
			Gdk.threads_leave()
		else:		
			name_of_the_program=self.entry_program_name.get_text()
			name_of_the_program=name_of_the_program.strip()
			
			if CustomCsv.exists_in_column(programs_to_install_path,0,name_of_the_program):
				Gdk.threads_enter()
				CustomGtk.info(self.window_add_programs,PROGRAM_ALREDY_IN_LIST,None)
				Gdk.threads_leave()
				CLEAR_gui_add_programs_to_install(self)
			else:
				CustomOs.MakeTreeOfDirs(PROGRAM_DATA.format(self.Data_Location))
				
				description=self.entry_program_description.get_text()
				comment=self.entry_program_comment.get_text()
				
				
				if self.check_existance_of_programs:
					(locked1,locked2,choice)=check_if_apt_is_locked(self,self.window_add_programs,TEXT_TRY_TO_UNLOCK_APT_IF_NOT_USE_BASH)
					if not locked1 and not locked2:
						programs_cache=os.popen("apt-cache search {0}".format(name_of_the_program))
						programs_cache=programs_cache.read()

						if name_of_the_program+" " in programs_cache and not re.search(r'\w'+name_of_the_program+" ",programs_cache):
							program_exists=True
						else:
							Gdk.threads_enter()
							if not CustomGtk.question(self.window_add_programs,TEXT_NO_PROGRAM_IN_REPOSITORY,None):
								Gdk.threads_leave()
								return
							Gdk.threads_leave()

				
				new_program=ProgramsToInstall(	name_of_the_program, 
												category, 
												description, 
												comment, 
												GET_ID_ProgramsToInstall(self))
													
													
													
					
				self.ProgramsToInstall_List.append(new_program)
				
				self.ListStore_ProgramsToInstall.append([	new_program.get_installed(),
															new_program.get_apt_name(),
															new_program.get_category(),
															new_program.get_description(),
															new_program.get_comment()])
				
									
				CustomCsv.add_row(	programs_to_install_path,
									name_of_the_program, 
									category, 
									description, 
									comment)
				
				
				CustomCsv.alph_organise_file(PROGRAMS_TO_INSTALL_CSV.format(self.Data_Location))
						
						
				CLEAR_gui_add_programs_to_install(self)
	
				CustomParser.write_value(self.Path_File_Config, self.Parser_Section,'program_list_modified','true')

				if os.path.exists(PROGRAM_LIST_BY_CATEGORY.format(self.Data_Location, category)):
					os.remove(PROGRAM_LIST_BY_CATEGORY.format(self.Data_Location, category))
					
				POPULATE_Combobox_get_category(self)
				POPULATE_Combobox_Category2(self)


def replace_strings_in_path(self,path):
	old_string=self.entry_replace_modify_path.get_text()
	new_string=self.entry_replace_modify_path2.get_text()
	
	if old_string:
		path=path.replace(old_string,new_string)
		return path

	return path
			

class Configuration:
	
	
	def init_session(self,GUISelf):

		if GUISelf.Current_User=="root":
			GUISelf.Path_File_Config=ROOT_CONFIG_PATH
			
			if CustomParser.check_value(GUISelf.Path_File_Config,'fresh-install_configuration','program_Data_Location'):
				GUISelf.Data_Location=CustomParser.get_value(GUISelf.Path_File_Config,'fresh-install_configuration','program_Data_Location')
			else:
				GUISelf.Data_Location=ROOT_MAIN_FOLDER
		
		else:
			GUISelf.Path_File_Config=USER_CONFIG_PATH
			
			if CustomParser.check_value(GUISelf.Path_File_Config,'fresh-install_configuration','program_Data_Location'):
				GUISelf.Data_Location=CustomParser.get_value(GUISelf.Path_File_Config,'fresh-install_configuration','program_Data_Location')
			else:
				GUISelf.Data_Location=USER_MAIN_FOLDER
		
		GUISelf.Parser_Section='fresh-install_configuration'
		
		if os.path.exists(GROUPED_CONFIG_PATH):
			GUISelf.Path_File_Config=GROUPED_CONFIG_PATH
			GUISelf.Data_Location=GROUPED_MAIN_FOLDER
			
			if CustomParser.get_value(GUISelf.Path_File_Config,'fresh-install_configuration','Merged_folders')!="true":
				GUISelf.Data_Location=GROUPED_MAIN_FOLDER
				
			if CustomParser.get_value(GUISelf.Path_File_Config,'fresh-install_configuration','Merged_configurations')=="true":
				GUISelf.Parser_Section='fresh-install_configuration'
			else:
				GUISelf.Parser_Section="fresh-install_"+GUISelf.Current_User+"_Configuration"
				
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'mac_adress'):
			MAC=CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'mac_adress')
			if int(MAC) != get_mac() and os.path.exists(TEMP_DATA.format(GUISelf.Data_Location)):
				CustomOs.DeleteFromDisk(TEMP_DATA.format(GUISelf.Data_Location))
		
	
	def init_gui(self,GUISelf):
	
		GUISelf.skip_ask_external_configuration=False
		GUISelf.Size_To_Actualize=0
			
		GUISelf.default_file_chooser_path=None
		GUISelf.default_folder_chooser_path=None

		LABEL_last_actualization(GUISelf)
		LABEL_user(GUISelf)
		LABEL_data(GUISelf,"Input","Output")
		
		GUISelf.label_config_location.set_label(GUISelf.Path_File_Config)
		GUISelf.label_Data_Location.set_label(GUISelf.Data_Location)
		GUISelf.label_folder_name.set_label(os.path.basename(GUISelf.Data_Location))


		#############################
		## Initializing the check boxs
		
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_file_size'):
			GUISelf.warning_file_size=float(CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_file_size'))
		else:
			GUISelf.warning_file_size=500
		GUISelf.spinbutton_file_size.set_value(float(GUISelf.warning_file_size))

			
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_folder_size'):
			GUISelf.warning_folder_size=float(CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_folder_size'))
		else:
			GUISelf.warning_folder_size=0.5
		GUISelf.spinbutton_folder_size.set_value(float(GUISelf.warning_folder_size))	
			
			
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_waiting_dialog'):
			GUISelf.warning_waiting_dialog=float(CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_waiting_dialog'))
		else:
			GUISelf.warning_waiting_dialog=0.1
		GUISelf.spinbutton_waiting_dialog.set_value(float(GUISelf.warning_waiting_dialog))	
			
			
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_renaming_files'):
			if CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_renaming_files')=="false":
				GUISelf.checkbutton_warn_rename.set_active(False)
				GUISelf.warning_renaming_files=False
		else:
			GUISelf.checkbutton_warn_rename.set_active(True)
			GUISelf.warning_renaming_files=True
			
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_installing_programs'):
			if CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_installing_programs')=="false":
				GUISelf.checkbutton_warn_install.set_active(False)
				GUISelf.warning_installing_programs=False
		else:
			GUISelf.checkbutton_warn_install.set_active(True)
			GUISelf.warning_installing_programs=True
				
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_replacing_files'):
			if CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_replacing_files')=="false":
				GUISelf.checkbutton_warn_rep_file.set_active(False)
				GUISelf.warning_replacing_files=False
		else:
			GUISelf.checkbutton_warn_rep_file.set_active(True)
			GUISelf.warning_replacing_files=True
			
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_replacing_folders'):
			if CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_replacing_folders')=="false":
				GUISelf.checkbutton_warn_rep_fol.set_active(False)
				GUISelf.warning_replacing_folders=False
		else:
			GUISelf.checkbutton_warn_rep_fol.set_active(True)
			GUISelf.warning_replacing_folders=True
			
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_replacing_files_and_folders'):
			if CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_replacing_files_and_folders')=="false":
				GUISelf.checkbutton_warn_rep_fileNfol.set_active(False)
				GUISelf.warning_replacing_files_and_folders=False
		else:
			GUISelf.checkbutton_warn_rep_fileNfol.set_active(True)
			GUISelf.warning_replacing_files_and_folders=True
			
			
		if CustomParser.check_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_existance_of_programs'):
			if CustomParser.get_value(GUISelf.Path_File_Config,GUISelf.Parser_Section,'warning_existance_of_programs')=="false":
				GUISelf.checkbutton_existance_of_programs.set_active(False)
				GUISelf.check_existance_of_programs=False
		else:
			GUISelf.checkbutton_existance_of_programs.set_active(True)
			GUISelf.check_existance_of_programs=True


		Initialize_Data_FilesToReplace(GUISelf)
		Populate_Files_To_Replace(GUISelf)
		
		Initialize_Data_FoldersToReplace(GUISelf)	
		Populate_FoldersToReplace(GUISelf)
			
		Initialize_Data_ProgramsToInstall(GUISelf)

		Initialize_Data_FilesToRename(GUISelf)
		
		POPULATE_Combobox_get_category(GUISelf)
		POPULATE_Combobox_Category2(GUISelf)


		if not os.path.exists(GUISelf.Path_File_Config):
				Gdk.threads_enter()
				GUISelf.window_setup.show()
				Gdk.threads_leave()
		else:
			Gdk.threads_enter()
			GUISelf.window_root.show()
			Gdk.threads_leave()
			Gdk.threads_enter()
			GUISelf.window_setup.hide()
			Gdk.threads_leave()


class Thread:

	def start_waiting(self,GUISelf,parent,func,args):
	
		GUISelf.the_sync_exited_normally=True
		GUISelf.progressbar.set_fraction(0.0)
		GUISelf.Synchronized_Size=0
		GUISelf.window_waiting.set_transient_for(parent)	

		if not "INSTALL" in str(func):
			GUISelf.label_data_being_copied.set_label(TEXT_DATA_BEING_COPIED)
			GUISelf.ened_start_waiting_message=TEXT_THE_DATA_HAS_BEEN_COPIED
		else:
			GUISelf.label_data_being_copied.set_label(TEXT_THE_PROGRAMS_ARE_BEING_INSTALLED)
			GUISelf.ened_start_waiting_message=TEXT_THE_INSTALLATION_HAS_FINISHED
		
		GUISelf.button_waiting.set_label(TEXT_CANCEL)
		GUISelf.button_waiting.set_image(GUISelf.icon_cancel)
		
		GUISelf.task=threading.Thread(target=func, args=args)
		GUISelf.thrd=threading.Thread(target=self.thread_start_waiting, args=[GUISelf])
		GUISelf.thrd.start()
		

	def thread_start_waiting(self,GUISelf):

		# clean the terminal
		command_on_VTE(GUISelf,'''printf "\\033c"''')
		
		Gdk.threads_enter()
		GUISelf.window_waiting.show_all()
		Gdk.threads_leave()
		
		Gdk.threads_enter()
		GUISelf.window_waiting.get_window().set_cursor(GUISelf.watch_cursor)
		Gdk.threads_leave()
		
		GUISelf.task.start()
		GUISelf.task.join()
		
		GLib.idle_add(self.end_of_start_waiting,GUISelf)
		
	
	def end_of_start_waiting(self,GUISelf):

		if GUISelf.the_sync_exited_normally:
			GUISelf.label_data_being_copied.set_label(GUISelf.ened_start_waiting_message)
			GUISelf.progressbar.set_fraction(1)
			
		GUISelf.button_waiting.set_label(TEXT_CLOSE)
		GUISelf.button_waiting.set_image(GUISelf.icon_close)
		
		Gdk.threads_enter()
		GUISelf.window_waiting.get_window().set_cursor(None)
		Gdk.threads_leave()
		
		if GUISelf.checkbutton_atom_close.get_active():
			GUISelf.window_waiting.hide()
			
			

	def set_watch_cursor(self,GUISelf,window,button,func,args):
	
		if button:
			button.set_sensitive(False)
		
		window.get_window().set_cursor(GUISelf.watch_cursor)

		GUISelf.task=threading.Thread(target=func, args=args)
		GUISelf.thrd=threading.Thread(target=self.thread_watch_cursor, args=[GUISelf,button,window])
		GUISelf.thrd.start()

	def thread_watch_cursor(self,GUISelf,button,window):
		
		GUISelf.task.start()		
		GUISelf.task.join()
		GLib.idle_add(self.end_of_watch_cursor,GUISelf,button,window)
		
		
	def end_of_watch_cursor(self,GUISelf,button,window):
	
		if button:
			button.set_sensitive(True)
		
		window.get_window().set_cursor(None)
	
	
class GUI(Gtk.Window):

	def __init__(self):
		
		self.Current_User=getpass.getuser()
		self.Script_Location=os.path.dirname(os.path.realpath(__file__))
		
		Configuration.init_session(self)
		
		Import_Data_From_Glade(self)

		self.watch_cursor=Gdk.Cursor(Gdk.CursorType.WATCH)
		
		self.icon_cancel=Gtk.Image()
		self.icon_cancel.set_from_stock(Gtk.STOCK_CANCEL, Gtk.IconSize.BUTTON)
			
		self.icon_close=Gtk.Image()
		self.icon_close.set_from_stock(Gtk.STOCK_CLOSE, Gtk.IconSize.BUTTON)
		
		self.right_click_menu=None  # this a dirty hack to make work the Gtk.Menu on Debian and Ubuntu

		INIT_ComboBoxes(self)
		
		Init_VTE_Terminal(self)
		
		INIT_TreeView_ProgramsToInstall(self)
		INIT_TreeView_FilesToReplace(self)
		INIT_TreeView_FoldersToReplace(self)
		INIT_TreeView_FilesToRename(self)

		threading.Thread(target=Configuration.init_gui, args=[self]).start()
		
	#############################
	#### window_setup ###########
	
	def on_button_external_profile_clicked(self, button, data=None):
		path=AskForFile(self,self.window_setup,"external_config")

		if path and not os.stat(path)[6]==0:
			if not CustomTextFile.CurrentUserCanWrite(path):
				CustomGtk.info(self.window_root,TEXT_ERROR,TEXT_NO_WRITE_PERMISSION_ON_CONF_FILE)
			else:
				self.skip_ask_external_configuration=True
				self.button_external_configuration.set_active(True)
				
				Thread.set_watch_cursor(self,self.window_setup,self.button_external_configuration,LOAD_external_config,[self,path])

				if not os.path.exists(self.Path_File_Config):
					self.window_setup.show()
				else:
					self.window_root.show()
					self.window_setup.hide()
						
	def on_button_go_profile_window_clicked(self, button, data=None):
		self.window_new_profile.show()
		self.window_setup.hide()
		self.checkbutton_local_profile.set_active(True)
		
		self.label_profile_help.set_label(TEXT_EXPLANATION_LOCAL_CONFIGURATION)
		self.label_profile_help.set_line_wrap(True)
		self.label_profile_help.set_justify(Gtk.Justification.FILL)
		
		if self.Current_User!="root":
			self.checkbutton_global_profile.set_active(False)
			self.checkbutton_merged_configurations.set_active(True)
			self.checkbutton_global_profile.set_inconsistent(True)
			self.checkbutton_merged_folders.set_inconsistent(True)
			self.checkbutton_merged_configurations.set_inconsistent(True)
	
	
	##############################
	#### window_new_profile ######
	
	def on_button_back_setup_clicked(self, button, data=None):
		self.window_new_profile.set_transient_for(self.window_setup)
		
		self.window_setup.show()
		self.window_new_profile.hide()
		
		
	def on_button_create_profile_clicked(self, button, data=None):
		
		if self.checkbutton_local_profile.get_active():
			if self.Current_User=="root":
				self.Data_Location=ROOT_MAIN_FOLDER
				self.Path_File_Config=ROOT_CONFIG_PATH
			else:
				self.Data_Location=USER_MAIN_FOLDER
				self.Path_File_Config=USER_CONFIG_PATH
				
			CustomOs.MakeTreeOfDirs(self.Data_Location)

		elif self.checkbutton_global_profile.get_active():
			if self.checkbutton_merged_folders.get_active():
				self.Data_Location=GROUPED_MAIN_FOLDER
			else:
				if self.Current_User=="root":
					self.Data_Location=ROOT_MAIN_FOLDER
				else:
					self.Data_Location=USER_MAIN_FOLDER
			
			CustomOs.MakeTreeOfDirs(self.Data_Location)
			
			self.Path_File_Config=GROUPED_CONFIG_PATH	
			if self.checkbutton_merged_configurations.get_active():
				CustomParser.write_value(self.Path_File_Config,'fresh-install_configuration','Merged_configurations',"true")
			else:
				CustomParser.write_value(self.Path_File_Config,'fresh-install_configuration','Merged_configurations',"false")
				
			if self.checkbutton_merged_folders.get_active():
				CustomParser.write_value(self.Path_File_Config,'fresh-install_configuration','Merged_folders',"true")
			else:
				CustomParser.write_value(self.Path_File_Config,'fresh-install_configuration','Merged_folders',"false")
				
			os.system("chmod -R 0777 '"+self.Path_File_Config+"'")

		CustomParser.write_value(self.Path_File_Config,'fresh-install_configuration','program_Data_Location',self.Data_Location)

		self.window_root.set_transient_for(self.window_new_profile)
		self.window_root.show()
		self.window_new_profile.hide()
		Configuration.init_session(self)

	def on_checkbutton_local_profile_clicked(self, checkbutton, data=None):
		if self.Current_User=="root":
			if self.checkbutton_local_profile.get_active() and self.checkbutton_global_profile.get_active():
				self.checkbutton_global_profile.set_active(False)
				
				
			self.checkbutton_merged_folders.set_inconsistent(True)
			self.checkbutton_merged_configurations.set_inconsistent(True)
		
		if self.Current_User!="root":
			self.checkbutton_local_profile.set_active(True)
	
		LABEL_new_profile_help(self)
		
		
	def on_checkbutton_global_profile_clicked(self, checkbutton, data=None):
		
		if self.Current_User=="root":
			if self.checkbutton_local_profile.get_active() and self.checkbutton_global_profile.get_active():
				self.checkbutton_local_profile.set_active(False)
				
				if self.checkbutton_local_profile.get_active():
					self.checkbutton_merged_folders.set_inconsistent(True)
					self.checkbutton_merged_configurations.set_inconsistent(True)
				else:
					self.checkbutton_merged_folders.set_inconsistent(False)
					self.checkbutton_merged_configurations.set_inconsistent(False)
					
		LABEL_new_profile_help(self)

	def on_checkbutton_merged_folders_clicked(self, checkbutton, data=None):
		if not self.checkbutton_merged_folders.get_active() and not self.checkbutton_merged_configurations.get_active():
			self.checkbutton_merged_folders.set_active(True)
			self.checkbutton_local_profile.set_active(True)

		LABEL_new_profile_help(self)

	def on_checkbutton_merged_configurations_clicked(self, checkbutton, data=None):
		if not self.checkbutton_merged_folders.get_active() and not self.checkbutton_merged_configurations.get_active():
			self.checkbutton_merged_configurations.set_active(True)
			self.checkbutton_local_profile.set_active(True)
		
		LABEL_new_profile_help(self)
		
	#####################
	#### window_root ####
	
	def on_root_window_destroy(self, object, data=None):
		Gtk.main_quit()
	
	# WINDOW ROOT: Action Buttons
	
	def on_button_reset_clicked(self, button, data=None):
		DELETE_all_stored_data(self)
		
	def on_button_about_clicked(self, menuitem, data=None):
		self.response=self.window_about.run()
		self.window_about.hide()

	def on_button_external_configuration_clicked(self, button, data=None):

		if not self.button_external_configuration.get_active():
			Configuration.init_session(self)
			if not os.path.exists(self.Path_File_Config) or os.stat(self.Path_File_Config)[6]==0:
				self.window_root.hide()
				self.window_setup.show()
			else:
				Thread.set_watch_cursor(self,self.window_root,self.button_external_configuration,Configuration.init_gui,[self])
		else:
			if not self.skip_ask_external_configuration:
				path=AskForFile(self,self.window_root,"external_config")
				if not path or os.stat(path)[6]==0:
					self.button_external_configuration.set_active(False) 
				else:
					if not CustomTextFile.CurrentUserCanWrite(path):
						self.button_external_configuration.set_active(False)
						CustomGtk.info(self.window_root,TEXT_ERROR,TEXT_NO_WRITE_PERMISSION_ON_CONF_FILE)
					else:
						Thread.set_watch_cursor(self,self.window_root,self.button_external_configuration,LOAD_external_config,[self,path])
				
				self.skip_ask_external_configuration=False


	def on_button_actualize_clicked(self, menuitem, data=None):
		
		files_to_replace=FILES_TO_REPLACE_CSV.format(self.Data_Location)
		folder_to_replace=FOLDERS_TO_REPLACE_CSV.format(self.Data_Location)
		current_mac_adress=get_mac()
		
		
		if (not os.path.exists(files_to_replace) or os.stat(files_to_replace)[6]==0) and (not os.path.exists(folder_to_replace) or os.stat(folder_to_replace)[6]==0):
			CustomGtk.info(self.window_root,TEXT_NO_DATA_TO_ACTUALIZE,TEXT_NEED_TO_SELECT_FILESFOLDERS_BEFORE)
		else:
			"""
				Prevent the user of a synchronization if he it is not in the root computer
			"""
			MAC=CustomParser.get_value(self.Path_File_Config,self.Parser_Section,'mac_for_files_and_folders')
			if MAC and int(MAC)!=current_mac_adress:
				if not CustomGtk.question(self.window_root,TEXT_DETECT_ANOTHER_COMPUTER_WANT_TO_ACTUALIZE,None):
					return
				else:
					CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'mac_for_files_and_folders',current_mac_adress)
			
			"""
				Launch the synchronization
			"""
			if GET_SIZE_ACTUALIZE_files_and_folders(self) >= (self.warning_waiting_dialog*1024*1024):			
				Thread.start_waiting(self,self.window_root,ACTUALIZE_files_and_folders,[self])
			else:
				self.Synchronized_Size=0
				threading.Thread(target=ACTUALIZE_files_and_folders, args=[self])
					

	def on_button_config_clicked(self, button, data=None):
		self.window_configuration.show()

	def on_button_files_to_replace_clicked(self, button, data=None):
		path=AskForFile(self,self.window_root,None)
		if path:
			ADD_files_to_replace(self,path)

	# WINDOW ROOT: Input Buttons
	
	def on_button_files_to_rename_clicked(self, button, data=None):
			
		path=AskForFile(self,self.window_root,"rename")
	
		if path:
			for file in self.classlist_files_to_rename:
				if file.get_computer_path() == path:
					self.Current_FilePath_To_Rename=None
					CustomGtk.info(self.window_root,TEXT_FILE_ALREADY_SELECTED,None)
					return
		else:
			return	
		

		self.entry_rename_file.set_text(os.path.basename(path))
		self.window_new_name.show()
		self.Current_FilePath_To_Rename=path

			
	
	def on_button_folders_to_replace_clicked(self, button, data=None):
		folder_path=AskForFolder(self,self.window_root)
		
		if folder_path:
			ADD_folders_to_replace(self,folder_path)
		
		
	def on_button_programs_to_install_clicked(self, button, data=None):
		self.window_add_programs.show()
		self.window_root.hide()
	
	# WINDOW ROOT: Output Buttons
	
	def on_button_home_clicked(self, menuitem, data=None):
		if os.path.exists(self.Data_Location):
			os.system("exo-open '"+self.Data_Location+"'")
		else:
			print 'the data folder dont exists'
		
	def on_button_GUI_rename_files_clicked(self, button, data=None):	
		self.window_rename.show()
		self.window_root.hide()
		
	def on_button_GUI_files_and_folders_clicked(self, button, data=None):
		self.window_replace.show()
		self.window_root.hide()

	def on_button_GUI_install_programs_clicked(self, button, data=None):
		self.window_install_programs.show()
		self.window_root.hide()
		
	##############################
	#### window_configuration ####
	
	def on_button_change_data_clicked(self, button, data=None):
		path=AskForFolder(self,self.window_configuration)
		if path:
			old_Data_Location=self.Data_Location
			folder_name=os.path.basename(old_Data_Location)
			os.system("rsync -av '"+old_Data_Location+"' '"+path+"'")
			CustomOs.DeleteFromDisk(old_Data_Location)
			self.Data_Location=path+"/"+folder_name
			CustomParser.write_value(self.Path_File_Config,'fresh-install_configuration','program_Data_Location',self.Data_Location)
			self.label_Data_Location.set_label(self.Data_Location)
			
	def on_button_change_folder_name_clicked(self, button, data=None):
		self.window_new_name1.show()
		self.entry_rename_file1.set_text(os.path.basename(self.Data_Location))
		
	def	on_button_close7_clicked(self, button, data=None):
		self.window_configuration.hide()
		self.window_root.show()
	
	def on_spinbutton_file_size_value_changed(self, spinwidget, data=None):
		self.warning_file_size=self.spinbutton_file_size.get_value()
		CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_file_size',self.warning_file_size)

	def	on_spinbutton_folder_size_value_changed(self, spinwidget, data=None):
		self.warning_folder_size=self.spinbutton_folder_size.get_value()
		CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_folder_size',self.warning_folder_size)
	
	def on_spinbutton_waiting_dialog_value_changed(self, spinwidget, data=None):
		self.warning_waiting_dialog=self.spinbutton_waiting_dialog.get_value()
		CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_waiting_dialog',self.warning_waiting_dialog)
		
	def on_checkbutton_warn_rename_clicked(self, checkbutton, data=None):
		if self.checkbutton_warn_rename.get_active():
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_renaming_files',"true")
			self.warning_renaming_files=True
		else:
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_renaming_files',"false")
			self.warning_renaming_files=False
		
	def on_checkbutton_warn_install_clicked(self, checkbutton, data=None):
		if self.checkbutton_warn_install.get_active():
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_installing_programs',"true")
			self.warning_installing_programs=True
		else:
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_installing_programs',"false")
			warning_installing_programs=False
		
	def on_checkbutton_warn_rep_file_clicked(self, checkbutton, data=None):
		if self.checkbutton_warn_rep_file.get_active():
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_replacing_files',"true")
			self.warning_replacing_files=True
		else:
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_replacing_files',"false")
			self.warning_replacing_files=False
		
	def on_checkbutton_warn_rep_fol_clicked(self, checkbutton, data=None):
		if self.checkbutton_warn_rep_fol.get_active():
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_replacing_folders',"true")
			self.warning_replacing_folders=True
		else:
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_replacing_folders',"false")
			warning_replacing_folders=False
		
	def on_checkbutton_warn_rep_fileNfol_clicked(self, checkbutton, data=None):
		if self.checkbutton_warn_rep_fileNfol.get_active():
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_replacing_files_and_folders',"true")
			self.warning_replacing_files_and_folders=True
		else:
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_replacing_files_and_folders',"false")
			self.warning_replacing_files_and_folders=False
	
	def on_checkbutton_atomat_close_waiting_clicked(self, checkbutton, data=None):
		if self.checkbutton_atomat_close_waiting.get_active():
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'autom_close_waiting_dialog',"true")
			self.checkbutton_atom_close.set_active(True)
		else:
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'autom_close_waiting_dialog',"false")
			self.checkbutton_atom_close.set_active(False)
	
	def on_checkbutton_existance_of_programs_clicked(self, checkbutton, data=None):
		if self.checkbutton_existance_of_programs.get_active():
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_existance_of_programs',"true")
			self.checkbutton_existance_of_programs.set_active(True)
			self.check_existance_of_programs=True
		else:
			CustomParser.write_value(self.Path_File_Config,self.Parser_Section,'warning_existance_of_programs',"false")
			self.checkbutton_existance_of_programs.set_active(False)
			self.check_existance_of_programs=False
	
		## Window new name1 ##
	def on_button_new_name1_clicked(self, button, data=None):
		old_Data_Location=self.Data_Location
		parent_path=os.path.dirname(self.Data_Location)
		new_name=self.entry_rename_file1.get_text()
		
		if "/" in new_name:
			CustomGtk.info(self.window_configuration,TEXT_UNIX_DONT_ALLOW_SLASH,None)
		else:
			if not  CustomOs.NameExistsInFolder(parent_path,new_name):
			
				self.Data_Location=parent_path+"/"+new_name
				
				folder_name=os.path.basename(self.Data_Location)
				self.label_folder_name.set_label(folder_name)
				CustomParser.write_value(self.Path_File_Config,'fresh-install_configuration','program_Data_Location',self.Data_Location)
				

				self.label_Data_Location.set_label(self.Data_Location)
				
				os.rename(old_Data_Location,self.Data_Location)
				
				if os.path.exists(self.Data_Location+"/fresh-install.ini"):
					self.Path_File_Config=self.Data_Location+"/fresh-install.ini"
					self.label_config_location.set_label(self.Path_File_Config)
				
				
				
				self.window_new_name1.hide()
			
			
	def on_cancel_new_name1_clicked(self, button, data=None):
		self.window_new_name1.hide()
	
	#############################################
	#### window_add_programs ####################
	
	def on_button_program_list2_clicked(self, button, data=None):

		Populate_ProgramsToInstall_From(self,"combobox_category")

		if CustomGtk.ListStoreNbRows(self.ListStore_ProgramsToInstall) <= 0:
			CustomGtk.info(self.window_add_programs,TEXT_THE_LIST_IS_EMPTY,None)
		else:
			self.window_list_programs.set_transient_for(self.window_add_programs)
			self.window_list_programs.show()
		
		
		
	def on_button_cancel_add_program_clicked(self, button, data=None):
		CLEAR_gui_add_programs_to_install(self)
		
	def on_button_add_program_clicked(self, button, data=None):
		#ADD_programs_to_install(self)
		Thread.set_watch_cursor(self,self.window_add_programs,self.button_add_program,ADD_programs_to_install,[self])
		
	def on_button_return_to_menu_clicked(self, button, data=None):
		(x, y)=self.window_add_programs.get_position()
		x,y=x-86,y-19
		self.window_add_programs.hide()
		self.window_root.move(x, y)
		self.window_root.show()
		
		
			###############################
			### window_list_programs ######
			
	def on_button_close_window_list_programs_clicked(self, button, data=None):
		self.window_list_programs.hide()
		
	#######################
	#### window_rename ####
	
	def on_button_list_files_to_rename_clicked(self, widget):
		Populate_FilesToRename(self)
		
		if CustomGtk.ListStoreNbRows(self.ListStore_FilesToRename) <= 0:
			CustomGtk.info(self.window_replace,TEXT_YOU_HAVENT_RENAMED_ANY_FILE,None)
		else:
			self.window_list_files_to_rename.show()
		
	def on_checkbutton_rename_if_not_clicked(self, checkbutton, data=None):
		if self.checkbutton_rename_if_not.get_active() and self.checkbutton_rename_if.get_active():
			self.checkbutton_rename_if.set_active(False)
		
	def on_checkbutton_rename_if_clicked(self, checkbutton, data=None):	
		if self.checkbutton_rename_if_not.get_active() and self.checkbutton_rename_if.get_active():
			self.checkbutton_rename_if_not.set_active(False)
			
	def on_button_rename_files_clicked(self, button, data=None):
		
		Populate_FilesToRename(self)	
				
		if len(self.classlist_files_to_rename) == 0:	
			CustomGtk.info(self.window_replace,TEXT_YOU_HAVENT_RENAMED_ANY_FILE,None)
				
		elif CustomGtk.ListStoreNbRows(self.ListStore_FilesToRename) == 0:
			CustomGtk.info(self.window_replace,TEXT_NO_FILES_TO_RENAME,None)
		
		else:
			Thread.set_watch_cursor(self,self.window_rename,self.button_replace_files,RENAME_files,[self])

	
	def on_button_exit5_clicked(self, button, data=None):
		Gtk.main_quit()
		CustomOs.KillChildTree()
		exit()
		
	def on_button_menu5_clicked(self, button, data=None):
		(x, y)=self.window_rename.get_position()
		x,y=x-95,y-106
		self.window_rename.hide()
		self.window_root.move(x, y)
		self.window_root.show()
		
			## Window new name ##
			
	def on_cancel_new_name_clicked(self, button, data=None):
		self.window_new_name.hide()

	def on_button_new_name_clicked(self, button, data=None):
	
		new_name=self.entry_rename_file.get_text()
		parent_path=os.path.dirname(self.Current_FilePath_To_Rename)
		
		
		if "/" in new_name:
			CustomGtk.info(self.window_root,TEXT_UNIX_DONT_ALLOW_SLASH,None)
		elif "|" in new_name:
			CustomGtk.info(self.window_root,TEXT_PROGRAMS_USES_THIS_STRING_AS_SEPARATOR,None)
		else:
			if CustomOs.NameExistsInFolder(parent_path,new_name):
				CustomGtk.info(self.window_new_name,TEXT_ALREADY_A_FILE_WITH_THIS_NAME_ON_THE_FOLDER,None)
			else:
				ADD_files_to_rename(self, self.Current_FilePath_To_Rename, new_name)
				self.window_new_name.hide()


		###################################
		## window_list_files_to_rename  ###


	def on_button_close_window_files_to_rename_clicked(self, button, data=None):
		self.window_list_files_to_rename.hide()

	########################
	#### window_replace ####
	
	def on_checkbutton_replace_if_clicked(self, checkbutton, data=None):
		if self.checkbutton_replace_if_not.get_active() and self.checkbutton_replace_if.get_active():
			self.checkbutton_replace_if_not.set_active(False)
		
	def on_checkbutton_replace_if_not_clicked(self, checkbutton, data=None):
		if self.checkbutton_replace_if_not.get_active() and self.checkbutton_replace_if.get_active():
			self.checkbutton_replace_if.set_active(False)
	
	def on_button_list_files_to_replace_clicked(self, button, data=None):
		Populate_Files_To_Replace(self)
		
		if CustomGtk.ListStoreNbRows(self.ListStore_FilesToReplace) <= 0:
			CustomGtk.info(self.window_replace,TEXT_THE_LIST_IS_EMPTY,None)
		else:
			self.window_list_files_to_replace.show()
		
	def on_button_list_folders_to_replace_clicked(self, button, data=None):
		Populate_FoldersToReplace(self)
		
		if CustomGtk.ListStoreNbRows(self.ListStore_FoldersToReplace) <= 0:
			CustomGtk.info(self.window_replace,TEXT_THE_LIST_IS_EMPTY,None)
		else:
			self.window_list_folders_to_replace.show()

	def on_button_replace_files_clicked(self, button, data=None):

		if not CustomGtk.ListStoreNbRows(self.ListStore_FilesToReplace) > 0:
			CustomGtk.info(self.window_replace,TEXT_NO_FILES_TO_REPLACE,TEXT_NEED_TO_SELECT_FILESFOLDERS_BEFORE)
		
		elif GET_SIZE_REPLACE_files(self) > (self.warning_waiting_dialog*1024*1024):
			Thread.start_waiting(self, self.window_replace, REPLACE_files, [self, False])
		else:
			Thread.set_watch_cursor(self, self.window_replace, self.button_replace_files ,REPLACE_files, [self, False])
			
	def on_button_replace_folders_clicked(self, button, data=None):	

		if not CustomGtk.ListStoreNbRows(self.ListStore_FoldersToReplace) > 0 :
			CustomGtk.info(self.window_replace,TEXT_NO_FOLDER_TO_REPLACE,TEXT_NEED_TO_SELECT_FILESFOLDERS_BEFORE)
	
		elif GET_SIZE_REPLACE_folders(self) > (self.warning_waiting_dialog*1024*1024):
			Thread.start_waiting(self, self.window_replace, REPLACE_folders, [self,False])
		else:
			Thread.set_watch_cursor(self, self.window_replace, self.button_replace_folders, REPLACE_folders,[self,False])		
				
	def on_button_replace_file_and_folders_clicked(self, button, data=None):
		
		if not CustomGtk.ListStoreNbRows(self.ListStore_FilesToReplace) > 0 or \
		   not CustomGtk.ListStoreNbRows(self.ListStore_FoldersToReplace) > 0:
				
			CustomGtk.info(self.window_replace,TEXT_TEXT_NO_FILES_OR_FOLDERS_TO_REPLACE,TEXT_NEED_TO_SELECT_FILESFOLDERS_BEFORE)
		
		else:
			total_size=GET_SIZE_REPLACE_folders(self)+GET_SIZE_REPLACE_files(self)
			max_size=(self.warning_waiting_dialog*1024*1024)
			
			if total_size >= max_size:
				Thread.start_waiting(self,self.window_replace,REPLACE_files,[self,False])
				Thread.start_waiting(self,self.window_replace,REPLACE_folders,[self,True])
			else:
				Thread.set_watch_cursor(self,self.window_replace,self.button_replace_files,REPLACE_files,[self, False])
				Thread.set_watch_cursor(self,self.window_replace,self.button_replace_folders,REPLACE_folders,[self,True])

	
	def on_button_exit4_clicked(self, button, data=None):
		Gtk.main_quit()
		CustomOs.KillChildTree()
		exit()
		
	def on_button_menu4_clicked(self, button, data=None):
		(x, y)=self.window_replace.get_position()
		x,y=x-57,y-95
		self.window_replace.hide()
		self.window_root.move(x, y)
		self.window_root.show()
		
		
				#################################
				#### window_list_files_to_replace
	
	def on_button_close_window_files_to_replace_clicked(self, button, data=None):
		self.window_list_files_to_replace.hide()
		
				#################################
				#### window_list_folders_to_replace
	
	def on_button_close_window_folders_to_replace_clicked(self, button, data=None):
		self.window_list_folders_to_replace.hide()	
	
	
	###########################################
	#### window_install_programs ####
	
	def on_button_program_list1_clicked(self, button, data=None):
		Populate_ProgramsToInstall_From(self,"combobox_category2")

		if CustomGtk.ListStoreNbRows(self.ListStore_ProgramsToInstall) <= 0:
			CustomGtk.info(self.window_install_programs,TEXT_THE_LIST_IS_EMPTY,None)
		else:
			self.window_list_programs.set_transient_for(self.window_install_programs)
			self.window_list_programs.show()
			
			
	def on_button_refresh_installed_programs_clicked(self, button, data=None):
		Thread.set_watch_cursor(self,self.window_list_programs,self.button_refresh_installed_programs,REFRESH_installed_programs,[self, False])
		
	def on_button_install_programs_clicked(self, button, data=None):
	
		box_category=CustomGtk.retrive_combobox_value(self.combobox_category2,False)
		
		
		list_path=PROGRAMS_TO_INSTALL_CSV.format(self.Data_Location)
		
		if not os.path.exists(list_path):
			CustomGtk.info(self.window_install_programs,TEXT_NO_PROGRAM_TO_INSTALL,TEXT_YOU_NEED_TO_ADD_PROGRAMS)
		else:
			if not self.Current_User=="root":
				CustomGtk.info(self.window_install_programs,TEXT_MUST_BE_ROOT_TO_INSTALL,TEXT_IF_NORMAL_USER_OPEN_AS_ROOT)
			else:
				
				(locked1,locked2,choice)=check_if_apt_is_locked(self,self.window_install_programs,TEXT_TRY_TO_UNLOCK_APT_IF_NOT_USE_BASH)
				
				if not locked1 and not locked2:
					if box_category==TEXT_SELECT_ALL_THE_CATEGORIES:

						choice=False
						if self.warning_installing_programs:
							choice=CustomGtk.question(self.window_install_programs,TEXT_WARNING,TEXT_WILL_INSTALL_PROGRAMS_WANT_TO_PROCEED)
							
						if choice or not self.warning_installing_programs:
							Thread.start_waiting(self,self.window_install_programs,INSTALL_programs,[self])
					else:
						if not CustomCsv.exists_in_column(list_path,1,box_category):
							CustomGtk.info(self.window_install_programs,TEXT_NO_PROGRAM_ON_THIS_CATEGORY, None)
						else:
							choice=False
							if self.warning_installing_programs:
								choice=CustomGtk.question(self.window_install_programs,TEXT_WARNING,TEXT_WILL_INSTALL_PROGRAMS_WANT_TO_PROCEED)
							
							if choice or not self.warning_installing_programs:
								Thread.start_waiting(self,self.window_install_programs,INSTALL_programs,[self])


	def on_button_generate_bash_clicked(self, button, data=None):
		
		box_category=CustomGtk.retrive_combobox_value(self.combobox_category2,False)
		list_path=PROGRAMS_TO_INSTALL_CSV.format(self.Data_Location)
		
		if not os.path.exists(list_path):
			CustomGtk.info(self.window_install_programs,TEXT_NO_PROGRAM_TO_INSTALL,TEXT_YOU_NEED_TO_ADD_PROGRAMS)
		else:
			folder_path=AskForFolder(self,self.window_install_programs)
			if folder_path:
				if not CustomTextFile.CurrentUserCanWrite(folder_path):
					CustomGtk.info(self.window_install_programs,TEXT_ERROR,TEXT_NEED_WRITE_PERMISSION_ON_FOLDER_TO_CREATE_IT)
				else:
					if box_category==TEXT_SELECT_ALL_THE_CATEGORIES:
						GENERATE_installation_bash_script(self,folder_path)
					else:
						if not CustomCsv.exists_in_column(list_path,1,box_category):
							CustomGtk.info(self.window_install_programs,TEXT_NO_PROGRAM_ON_THIS_CATEGORY, None)
						else:
							GENERATE_installation_bash_script(self,folder_path)

		
		
	def on_button_exit3_clicked(self, menuitem, data=None):
		Gtk.main_quit()
		CustomOs.KillChildTree()
		exit()
		
	def on_button_menu_2_clicked(self, menuitem, data=None):
		(x, y)=self.window_install_programs.get_position()
		self.window_install_programs.hide()
		x,y=x-55,y-83
		self.window_root.move(x, y)
		self.window_root.show()
		
	########################
	#### WINDOW Waiting ####
		
	def on_button_waiting_clicked(self, button, data=None):
		text=self.button_waiting.get_label()
		
		if text==TEXT_CANCEL:
			self.window_waiting.get_window().set_cursor(None)
			self.button_waiting.set_label(TEXT_CLOSE)
			self.button_waiting.set_image(self.icon_close)
			
			self.the_sync_exited_normally=False

			CustomOs.KillChildTree()
			
			# This kill all the rsync process.. it would be better to detect the rsync process created by the program and only kill
			# those.
			os.system("pkill rsync")
			
			if self.ened_start_waiting_message==TEXT_THE_DATA_HAS_BEEN_COPIED:
				self.label_data_being_copied.set_label(TEXT_TRANSFER_CANCELLED)
			else:
				self.label_data_being_copied.set_label(TEXT_INSTALLATION_CANCELLED)
				command_on_VTE(self, '''dpkg --configure -a ''')

		elif text==TEXT_CLOSE:
			self.window_waiting.hide()
			
			
		self.window_root.get_window().set_cursor(None)
		self.button_actualize.set_sensitive(True)
		
	def expander_waiting_callback(self, expander):
		(x,y)=self.window_waiting.get_size()
		if self.expander_waiting.get_expanded():
			None
			# This is not working..
			#print 'small'
			#self.window_waiting.resize(x,y)
			#self.window_waiting.set_size_request(-1, -1)
		else:
			#self.window_waiting.set_size_request(-1, -1)
			None
		
	def on_checkbutton_atom_close_clicked(self, checkbutton, data=None):
		if self.checkbutton_atom_close.get_active():
			self.checkbutton_atomat_close_waiting.set_active(True)
			if self.progressbar.get_fraction()==1:
				self.window_waiting.hide()
		else:
			self.checkbutton_atomat_close_waiting.set_active(False)
		
		
		########################
		#### Global Buttons ####
		
	def on_button_exit_clicked(self, button, data=None):
		Gtk.main_quit()
		CustomOs.KillChildTree()
		exit()
		
	def treeview_right_click_menu(self, treeview, event):
		if event.button == 3: # right click
			path, column, __, __ = treeview.get_path_at_pos(int(event.x), int(event.y))
			
			self.right_click_menu=None			# this is a dirty hack to clear the items of the menu
			self.right_click_menu=Gtk.Menu()
			
			deleteitem=Gtk.ImageMenuItem('Delete')
			self.right_click_menu.append(deleteitem)
			
			if treeview == self.treeview_ProgramsToInstall:
				deleteitem.connect("activate", DELETE_program_to_install, self)
			elif treeview == self.treeview_FoldersToReplace:
				deleteitem.connect("activate", DELETE_folders_to_replace, self)
			elif treeview == self.treeview_FilesToReplace:
				deleteitem.connect("activate", DELETE_files_to_replace, self)
			elif treeview == self.treeview_FilesToRename:
				deleteitem.connect("activate", DELETE_files_to_rename, self)
				
				
			icon_delete=Gtk.Image()
			icon_delete.set_from_stock(Gtk.STOCK_MISSING_IMAGE, Gtk.IconSize.MENU)
			deleteitem.set_image(icon_delete)
			
			self.right_click_menu.show_all()
			self.right_click_menu.popup(None, None, 
					   lambda menu, data: (event.get_root_coords()[0],
										  event.get_root_coords()[1], True),
					   None, event.button, event.time)
					   
					   
			return True



if __name__ == "__main__":
	
	GObject.threads_init()
	Gdk.threads_init()
	
	Configuration=Configuration()
	Thread=Thread()
	main=GUI()
	
	Gtk.main()
